      PROGRAM lmdz1d

      use phys_state_var_mod
      use comgeomphy
      use dimphy
      use surface_data, only : type_ocean,ok_veget
      use pbl_surface_mod, only : pbl_surface_init
      use fonte_neige_mod, only : fonte_neige_init

      use infotrac ! new

      implicit none
#include "dimensions.h"
#include "YOMCST.h"
#include "temps.h"
#include "control.h"
#include "clesphys.h"
#include "dimsoil.h"
#include "indicesol.h"

#include "comvert.h"

!=====================================================================
! DECLARATIONS
!=====================================================================

!---------------------------------------------------------------------
!  Externals
!---------------------------------------------------------------------
      external q_sat
      real q_sat

!---------------------------------------------------------------------
!  Arguments d' initialisations de la physique (USER DEFINE)
!---------------------------------------------------------------------

      integer, parameter :: ngrid=1
      real :: airefi   = 1.
      real :: zcufi    = 1.
      real :: zcvfi    = 1.

      real :: nat_surf
      logical :: ok_flux_surf
      real :: fsens
      real :: flat
      real :: tsurf
      real :: rugos
      real :: qsol(1:2)
      real :: qsurf
      real :: psurf
      real :: zsurf
      real :: albedo

      real :: time     = 0.
      real :: time_ini
      real :: xlat 
      real :: xlon 
      real :: wtsurf 
      real :: wqsurf 
      real :: restart_runoff 
      real :: xagesno 
      real :: qsolinp 
      real :: zpicinp 

      real :: fnday 
      real :: day 
      real :: paire    = 1.     ! aire de la maille
      real :: day1
      common /flux_arp/fsens,flat,ok_flux_surf

!---------------------------------------------------------------------
!  Declarations related to forcing and initial profiles 
!---------------------------------------------------------------------

        integer :: kmax = llm
        integer nlev_max
        parameter (nlev_max = 100)
        real timestep, frac, timeit
        real height(nlev_max),thlprof(nlev_max),qtprof(nlev_max),
     .              uprof(nlev_max),vprof(nlev_max),e12prof(nlev_max),
     .              ugprof(nlev_max),vgprof(nlev_max),wfls(nlev_max),
     .              dqtdxls(nlev_max),dqtdyls(nlev_max),
     .              dqtdtls(nlev_max),thlpcar(nlev_max)

        real    :: fff
        integer :: forcing_type
        logical :: forcing_les     = .false.
        logical :: forcing_radconv = .false.
        logical :: forcing_toga    = .false.
        logical :: forcing_GCM2SCM = .false.
        logical :: forcing_GCSSold = .false.
        integer :: type_ts_forcing ! 0 = SST constant; 1 = SST read from a file (cf read_tsurf1d.F)

! Forcing_toga
        character*80 :: fich_toga
!        integer nlev_prof
!        parameter (nlev_prof = 41)
        integer nlev_toga, nt_toga
        parameter (nlev_toga=41, nt_toga=480)
        integer year_ini_toga, day_ini_toga
        parameter (year_ini_toga=1992) 
        parameter (day_ini_toga=306)  ! 306 = 1erNov1992
        real dt_toga
        parameter (dt_toga=6.*3600.)

        integer nn
        integer it_toga1, it_toga2
        real time_toga1,time_toga2

        real ts_toga(nt_toga)
        real plev_toga(nlev_toga,nt_toga),w_toga(nlev_toga,nt_toga)
        real t_toga(nlev_toga,nt_toga),q_toga(nlev_toga,nt_toga)
        real u_toga(nlev_toga,nt_toga),v_toga(nlev_toga,nt_toga)
        real ht_toga(nlev_toga,nt_toga),vt_toga(nlev_toga,nt_toga)
        real hq_toga(nlev_toga,nt_toga),vq_toga(nlev_toga,nt_toga)

        real ts_prof
        real plev_prof(nlev_toga),w_prof(nlev_toga)
        real t_prof(nlev_toga),q_prof(nlev_toga)
        real u_prof(nlev_toga),v_prof(nlev_toga)
        real ht_prof(nlev_toga),vt_prof(nlev_toga)
        real hq_prof(nlev_toga),vq_prof(nlev_toga)

        real plev_mod(llm),w_mod(llm), t_mod(llm),q_mod(llm)
        real u_mod(llm),v_mod(llm), ht_mod(llm),vt_mod(llm)
        real hq_mod(llm),vq_mod(llm)

        real ts_cur
        common /sst_forcing/ts_cur ! also in read_tsurf1d.F

! Forcing_GCSSold
        character*80 :: fich_gcssold_ctl
        character*80 :: fich_gcssold_dat
        real  ht_gcssold(llm),hq_gcssold(llm),hw_gcssold(llm)
        real  hu_gcssold(llm)
        real  hv_gcssold(llm)
        real  hthturb_gcssold(llm)
        real  hqturb_gcssold(llm)
        real  Ts_gcssold
        real  dtime_frcg
        logical :: imp_fcg_gcssold =.false.
        logical :: ts_fcg_gcssold  =.false.
        logical :: Tp_fcg_gcssold  =.false. 
        logical :: Turb_fcg_gcssold=.false.
        common /turb_forcing/ dtime_frcg,
     $      Turb_fcg_gcssold, hthturb_gcssold, hqturb_gcssold

!---------------------------------------------------------------------
!  Declarations related to vertical discretization:
!---------------------------------------------------------------------
      real :: pzero=1.e5
      real :: play (llm),zlay (llm),sig_s(llm),plev(llm+1)
      real :: playd(llm),zlayd(llm)

!---------------------------------------------------------------------
!  Declarations related to variables
!---------------------------------------------------------------------

      integer :: iq
      real :: phi(llm)
      real :: teta(llm),temp(llm),u(llm),v(llm)
      real :: omega(llm+1),omega2(llm),rho(llm+1)
      real :: ug(llm),vg(llm),fcoriolis
      real :: du_phys(llm),dv_phys(llm),dt_phys(llm)
      real :: du_dyn(llm),dv_dyn(llm),dt_dyn(llm)
      real :: dt_cooling(llm),d_t_cool(llm),d_th_adv(llm)
      real :: dq_cooling(llm),d_q_cool(llm)
      real :: tmpvar(llm)
      real :: alpha

      REAL, ALLOCATABLE, DIMENSION(:,:):: q
      REAL, ALLOCATABLE, DIMENSION(:,:):: dq
      REAL, ALLOCATABLE, DIMENSION(:,:):: dq_dyn
      REAL, ALLOCATABLE, DIMENSION(:,:):: d_q_adv

!---------------------------------------------------------------------
!  Initialization of surface variables
!---------------------------------------------------------------------
      real :: run_off_lic_0(1)
      real :: fder(1),snsrf(1,nbsrf),qsurfsrf(1,nbsrf)
      real :: evap(1,nbsrf),frugs(1,nbsrf)
      real :: tsoil(1,nsoilmx,nbsrf)
      real :: agesno(1,nbsrf)

!---------------------------------------------------------------------
!  Call to phyredem
!---------------------------------------------------------------------
      logical :: ok_writedem =.true.
      
!---------------------------------------------------------------------
!  Call to physiq
!---------------------------------------------------------------------
      integer, parameter :: longcles=20
      logical :: firstcall=.true.
      logical :: lastcall=.false.
      real :: phis    = 0.0
      real :: clesphy0(longcles) = 0.0
      real :: dpsrf 

!---------------------------------------------------------------------
!  Initializations of boundary conditions 
!---------------------------------------------------------------------
      integer, parameter :: yd = 360
      real :: phy_nat (yd) = 0.0 ! 0=ocean libre,1=land,2=glacier,3=banquise
      real :: phy_alb (yd)  ! Albedo land only (old value condsurf_jyg=0.3)
      real :: phy_sst (yd)  ! SST (will not be used; cf read_tsurf1d.F)
      real :: phy_bil (yd) = 1.0 ! Ne sert que pour les slab_ocean
      real :: phy_rug (yd) ! Longueur rugosite utilisee sur land only
      real :: phy_ice (yd) = 0.0 ! Fraction de glace
      real :: phy_fter(yd) = 0.0 ! Fraction de terre
      real :: phy_foce(yd) = 0.0 ! Fraction de ocean
      real :: phy_fsic(yd) = 0.0 ! Fraction de glace
      real :: phy_flic(yd) = 0.0 ! Fraction de glace

!---------------------------------------------------------------------
!  Fichiers et d'autres variables
!---------------------------------------------------------------------
      integer :: ierr,k,l,i,it=1,mxcalc
      integer jjmp1
      parameter (jjmp1=jjm+1-1/jjm)
      INTEGER nbteta
      PARAMETER(nbteta=3)
      REAL dudyn(iim+1,jjmp1,llm)
      REAL PVteta(1,nbteta)
      INTEGER read_climoz

!=====================================================================
! INITIALIZATIONS 
!=====================================================================

! Initialization of Common turb_forcing
       dtime_frcg = 0.
       Turb_fcg_gcssold=.false.
       hthturb_gcssold = 0.
       hqturb_gcssold = 0.

!---------------------------------------------------------------------
! OPTIONS OF THE 1D SIMULATION (lmdz1d.def)
!---------------------------------------------------------------------

        open(1,file="lmdz1d.def",status='old',iostat=ierr)
        if (ierr /= 0) stop 'ERROR: lmdz1d.def does not exist'
        read (1,*)
        write(6,*) 'Options lmdz1d.def: '
        read (1,111) fff
        forcing_type=nint(fff)
        write(*,*) forcing_type
        read (1,111) time_ini   ! TU of beginning of the simulation (hours)
        write(*,*) time_ini
        read (1,111) xlat
        write(*,*) xlat
        read (1,111) xlon
        write(*,*) xlon
        read (1,111) nat_surf
        write(*,*) nat_surf
        read (1,111) tsurf
        write(*,*) tsurf
        read (1,111) psurf
        write(*,*) psurf
        read (1,111) zsurf
        write(*,*) zsurf
        read (1,111) rugos
        write(*,*) rugos
        read (1,111) wtsurf
        write(*,*) wtsurf
        read (1,111) wqsurf
        write(*,*) wqsurf
        read (1,111) albedo
        write(*,*) albedo
        read (1,111) xagesno
        write(*,*) xagesno
        read (1,111) restart_runoff
        write(*,*) restart_runoff
        read (1,111) qsolinp
        write(*,*) qsolinp
        read (1,111) zpicinp
        write(*,*) zpicinp
        IF (forcing_type .eq.40) THEN
         read (1,112) imp_fcg_gcssold
         write(*,*)   imp_fcg_gcssold
         read (1,112) ts_fcg_gcssold 
         write(*,*)   ts_fcg_gcssold 
         read (1,112) Tp_fcg_gcssold
         write(*,*)   Tp_fcg_gcssold
         read (1,112) Turb_fcg_gcssold
         write(*,*)   Turb_fcg_gcssold
        ENDIF
111     format(T18,f16.4,T30)
112     format(T18,l16,T30)
        close(1)
        write(*,*) 'lmdz1d.def lu'

! forcing_type defines the way the SCM is forced:
!forcing_type = 0 ==> forcing_les = .true.
!             initial profiles from file prof.inp.001
!             no forcing by LS convergence ; 
!             surface temperature imposed ;
!             radiative cooling may be imposed (iflag_radia=0 in physiq.def)
!forcing_type = 1 ==> forcing_radconv = .true.
!             idem forcing_type = 0, but the imposed radiative cooling 
!             is set to 0 (hence, if iflag_radia=0 in physiq.def, 
!             then there is no radiative cooling at all)
!forcing_type = 2 ==> forcing_toga = .true.
!             initial profiles from TOGA-COARE IFA files 
!             LS convergence and SST imposed from TOGA-COARE IFA files 
!forcing_type = 3 ==> forcing_GCM2SCM = .true.
!             initial profiles from the GCM output
!             LS convergence imposed from the GCM output
!forcing_type = 40 ==> forcing_GCSSold = .true.
!             initial profile from GCSS file
!             LS convergence imposed from GCSS file
!
      if (forcing_type .eq.0) THEN
       forcing_les = .true.
      elseif (forcing_type .eq.1) THEN
       forcing_radconv = .true.
      elseif (forcing_type .eq.2) THEN
       forcing_toga    = .true.
      elseif (forcing_type .eq.3) THEN
       forcing_GCM2SCM = .true.
      elseif (forcing_type .eq.40) THEN
       forcing_GCSSold = .true.
      else
       write (*,*) 'ERROR : unknown forcing_type ', forcing_type
       stop 'Forcing_type should be 0,1,2,3 or 40'
      ENDIF

! if type_ts_forcing=0, the surface temp of 1D simulation is constant in time
! (specified by tsurf in lmdz1d.def); if type_ts_forcing=1, the surface temperature
! varies in time according to a forcing (e.g. forcing_toga) and is passed to read_tsurf1d.F
! through the common sst_forcing.

        type_ts_forcing = 0
        if (forcing_toga) type_ts_forcing = 1

!---------------------------------------------------------------------
!  Definition of the run
!---------------------------------------------------------------------

      call conf_gcm( 99, .TRUE. , clesphy0 )
      fnday=nday
      annee_ref = anneeref
      day_ref = dayref
      day_ini = dayref
      itau_dyn = 0
      itau_phy = 0

      day = day_ini + time_ini/24. ! 1st day and initial time of the simulation 

!---------------------------------------------------------------------
! Initialization of dimensions, geometry and initial state
!---------------------------------------------------------------------

      call init_phys_lmdz(1,1,llm,1,1)
      call suphel
      call initcomgeomphy
      call infotrac_init 

      allocate(q(llm,nqtot)) 
      allocate(dq(llm,nqtot)) 
      allocate(dq_dyn(llm,nqtot)) 
      allocate(d_q_adv(llm,nqtot)) 

c
c   No ozone climatology need be read in this pre-initialization
c          (phys_state_var_init is called again in physiq)
      read_climoz = 0
c
      call phys_state_var_init(read_climoz)

      if (ngrid.ne.klon) then
         print*,'stop in inifis'
         print*,'Probleme de dimensions :'
         print*,'ngrid = ',ngrid
         print*,'klon  = ',klon
         stop
      endif

      qsol = qsolinp
      qsurf = q_sat(tsurf,psurf/100.)
      rlat=xlat
      rlon=xlon
      day1= day_ini
      time=day-int(day)
      ts_toga(1)=tsurf ! needed by read_tsurf1d.F
      rho(1)=psurf/(rd*tsurf*(1.+(rv/rd-1.)*qsurf)) 

      ok_flux_surf=.false.
      fsens=-wtsurf*rcpd*rho(1)
      flat=-wqsurf*rlvtt*rho(1)

! Vertical discretization and pressure levels at half and mid levels:

      pa   = 5e4
!!      preff= 1.01325e5
      preff = psurf
      call disvert(pa,preff,ap,bp,dpres,presnivs,nivsigs,nivsig)
      sig_s=presnivs/preff
      plev =ap+bp*psurf
      play = 0.5*(plev(1:llm)+plev(2:llm+1))
ccc      zlay=-rd*300.*log(play/psurf)/rg ! moved after reading profiles

      write(*,*) '***********************'
      do l = 1, llm
       write(*,*) 'l,play(l),presnivs(l): ',l,play(l),presnivs(l)
      enddo
      write(*,*) '***********************'

!=====================================================================
! EVENTUALLY, READ FORCING DATA :
!=====================================================================

!----------------------------------------------------------------------
! forcing_les = .T. : Impose a constant cooling 
! forcing_radconv = .T. : Pure radiative-convective equilibrium:
!----------------------------------------------------------------------

      if (forcing_les .or. forcing_radconv .or. forcing_GCSSold ) then

!----------------------------------------------------------------------
! Read profiles from files: prof.inp.001 and lscale.inp.001
! (repris de readlesfiles)
!----------------------------------------------------------------------

      call readprofiles(nlev_max,kmax,height,
     .           thlprof,qtprof,uprof,vprof,
     .           e12prof,ugprof,vgprof,
     .           wfls,dqtdxls,dqtdyls,dqtdtls,
     .           thlpcar) 

! compute altitudes of play levels.
      zlay(1) =zsurf +  rd*tsurf*(psurf-play(1))/(rg*psurf)
      do l = 2,llm
        zlay(l) = zlay(l-1)+rd*tsurf*(psurf-play(1))/(rg*psurf)
      enddo

!----------------------------------------------------------------------
! Interpolation of the profiles given on the input file to
! model levels
!----------------------------------------------------------------------
      zlay(1) = zsurf +  rd*tsurf*(psurf-play(1))/(rg*psurf)
      do l=1,llm
        ! Above the max altutide of the input file

        if (zlay(l)<height(kmax)) mxcalc=l

        frac = (height(kmax)-zlay(l))/(height (kmax)-height(kmax-1))
        teta(l) =thlprof(kmax)-frac*(thlprof(kmax)-thlprof(kmax-1))
        if (tp_fcg_gcssold) then ! pot. temp. in initial profile
          temp(l)=teta(l)*(play(l)/pzero)**rkappa
        else
          temp(l) = teta(l)
        endif
          print *,' temp,teta ',l,temp(l),teta(l)
        q(l,1)  = qtprof(kmax)-frac*( qtprof(kmax)- qtprof(kmax-1))
        u(l)    =  uprof(kmax)-frac*(  uprof(kmax)-  uprof(kmax-1))
        v(l)    =  vprof(kmax)-frac*(  vprof(kmax)-  vprof(kmax-1))
        ug(l)   = ugprof(kmax)-frac*( ugprof(kmax)- ugprof(kmax-1))
        vg(l)   = vgprof(kmax)-frac*( vgprof(kmax)- vgprof(kmax-1))
        omega(l)=   wfls(kmax)-frac*(   wfls(kmax)-   wfls(kmax-1))

        dq_dyn(l,1) = dqtdtls(kmax)-frac*(dqtdtls(kmax)-dqtdtls(kmax-1))
        dt_cooling(l)
     .          =thlpcar(kmax)-frac*(thlpcar(kmax)-thlpcar(kmax-1))
        do k=2,kmax
          frac = (height(k)-zlay(l))/(height(k)-height(k-1))
          if(l==1) print*,k,height(k),thlprof(k)
          if(zlay(l)>height(k-1).and.zlay(l)<height(k)) then
            teta(l) =thlprof(k)-frac*(thlprof(k)-thlprof(k-1))
            if (tp_fcg_gcssold) then ! pot. temp. in initial profile
              temp(l)=teta(l)*(play(l)/pzero)**rkappa
            else
              temp(l) = teta(l)
            endif
          print *,' temp,teta ',l,temp(l),teta(l)
            q(l,1)  = qtprof(k)-frac*( qtprof(k)- qtprof(k-1))
            u(l)    =  uprof(k)-frac*(  uprof(k)-  uprof(k-1))
            v(l)    =  vprof(k)-frac*(  vprof(k)-  vprof(k-1))
            ug(l)   = ugprof(k)-frac*( ugprof(k)- ugprof(k-1))
            vg(l)   = vgprof(k)-frac*( vgprof(k)- vgprof(k-1))
            omega(l)=   wfls(k)-frac*(   wfls(k)-   wfls(k-1))
            dq_dyn(l,1)=dqtdtls(k)-frac*(dqtdtls(k)-dqtdtls(k-1))
            dt_cooling(l)
     .              =thlpcar(k)-frac*(thlpcar(k)-thlpcar(k-1))
          elseif(zlay(l)<height(1)) then ! profils uniformes pour z<height(1)
            teta(l) =thlprof(1)
            if (tp_fcg_gcssold) then ! pot. temp. in initial profile
              temp(l)=teta(l)*(play(l)/pzero)**rkappa
            else
              temp(l) = teta(l)
            endif
            q(l,1)  = qtprof(1)
            u(l)    =  uprof(1)
            v(l)    =  vprof(1)
            ug(l)   = ugprof(1)
            vg(l)   = vgprof(1)
            omega(l)=   wfls(1)
            dq_dyn(l,1)  =dqtdtls(1)
            dt_cooling(l)=thlpcar(1)
          endif
        enddo 

        temp(l)=max(min(temp(l),350.),150.)
        rho(l)  = play(l)/(rd*temp(l)*(1.+(rv/rd-1.)*q(l,1)))
        if (l .lt. llm) then
          zlay(l+1) = zlay(l) + (play(l)-play(l+1))/(rg*rho(l))
        endif
        omega2(l)=-rho(l)*omega(l)
        omega(l)= omega(l)*(-rg*rho(l)) !en Pa/s
        if (l>1) then
        if(zlay(l-1)>height(kmax)) then
           omega(l)=0.0
           omega2(l)=0.0
        endif   
        endif
        if(q(l,1)<0.) q(l,1)=0.0
        q(l,2)  = 0.0
      enddo

      endif ! forcing_les .or. forcing_GCSSold

      if (forcing_GCSSold) then
       fich_gcssold_ctl = './forcing.ctl'
       fich_gcssold_dat = './forcing8.dat'
       call copie(llm,play,psurf,fich_gcssold_ctl)
       call get_uvd2(it,timestep,fich_gcssold_ctl,fich_gcssold_dat,
     :               ht_gcssold,hq_gcssold,hw_gcssold,
     :               hu_gcssold,hv_gcssold,
     :               hthturb_gcssold,hqturb_gcssold,Ts_gcssold,
     :               imp_fcg_gcssold,ts_fcg_gcssold,
     :               Tp_fcg_gcssold,Turb_fcg_gcssold)
       print *,' get_uvd2 -> hqturb_gcssold ',hqturb_gcssold
      endif ! forcing_GCSSold


!---------------------------------------------------------------------
! Forcing from TOGA-COARE experiment (Ciesielski et al. 2002) :
!---------------------------------------------------------------------

      if (forcing_toga) then

! read TOGA-COARE forcing (native vertical grid, nt_toga timesteps):
      fich_toga = './d_toga/ifa_toga_coare_v21_dime.txt'
      CALL read_togacoare(fich_toga,nlev_toga,nt_toga
     :         ,ts_toga,plev_toga,t_toga,q_toga,u_toga,v_toga,w_toga
     :         ,ht_toga,vt_toga,hq_toga,vq_toga)

       write(*,*) 'Forcing TOGA lu'

! time interpolation for initial conditions:
      write(*,*) 'AVT 1ere INTERPOLATION: day,day1 = ',day,day1
      CALL interp_toga_time(day,day1,annee_ref
     i             ,year_ini_toga,day_ini_toga,nt_toga,dt_toga,nlev_toga
     i             ,ts_toga,plev_toga,t_toga,q_toga,u_toga,v_toga,w_toga
     i             ,ht_toga,vt_toga,hq_toga,vq_toga
     o             ,ts_prof,plev_prof,t_prof,q_prof,u_prof,v_prof,w_prof
     o             ,ht_prof,vt_prof,hq_prof,vq_prof)

! vertical interpolation:
      CALL interp_toga_vertical(play,nlev_toga,plev_prof
     :         ,t_prof,q_prof,u_prof,v_prof,w_prof
     :         ,ht_prof,vt_prof,hq_prof,vq_prof
     :         ,t_mod,q_mod,u_mod,v_mod,w_mod
     :         ,ht_mod,vt_mod,hq_mod,vq_mod,mxcalc)
       write(*,*) 'Profil initial forcing TOGA interpole'

! initial and boundary conditions :
      tsurf = ts_prof
      write(*,*) 'SST initiale: ',tsurf
      do l = 1, llm
       temp(l) = t_mod(l)
       q(l,1) = q_mod(l)
       q(l,2) = 0.0
       u(l) = u_mod(l)
       v(l) = v_mod(l)
       omega(l) = w_mod(l)
       omega2(l)=omega(l)/rg*airefi ! flxmass_w calcule comme ds physiq
!?       rho(l)  = play(l)/(rd*temp(l)*(1.+(rv/rd-1.)*q(l,1)))
!?       omega2(l)=-rho(l)*omega(l)
       alpha = rd*temp(l)*(1.+(rv/rd-1.)*q(l,1))/play(l)
       d_th_adv(l) = alpha*omega(l)/rcpd-(ht_mod(l)+vt_mod(l))
       d_q_adv(l,1) = -(hq_mod(l)+vq_mod(l))
       d_q_adv(l,2) = 0.0
      enddo

      endif ! forcing_toga

      if (forcing_GCM2SCM) then
        write (*,*) 'forcing_GCM2SCM not yet implemented'
        stop 'in initialization'
      endif ! forcing_GCM2SCM

      ts_cur = tsurf ! SST used in read_tsurf1d

      print*,'mxcalc=',mxcalc
      print*,'zlay=',zlay(mxcalc)
      print*,'play=',play(mxcalc)

!=====================================================================
! Initialisation de la physique : 
!=====================================================================

!  Rq: conf_phys.F90 lit tous les flags de physiq.def; conf_phys appele depuis physiq.F
!
! day_step, iphysiq lus dans gcm.def ci-dessus
! timestep: calcule ci-dessous from rday et day_step
! ngrid=1
! llm: defini dans .../modipsl/modeles/LMDZ4/libf/grid/dimension
! rday: defini dans suphel.F (86400.)
! day_ini: lu dans run.def (dayref)
! rlat,rlon lus dans lmdz1d.def
! airefi,zcufi,zcvfi initialises au debut de ce programme
! rday,ra,rg,rd,rcpd declares dans YOMCST.h et calcules dans suphel.F

      day_step = day_step/float(iphysiq)
      timestep =rday/day_step
      dtime_frcg = timestep

      call iniphysiq(ngrid,llm,rday,day_ini,timestep,
     .     rlat,rlon,airefi,zcufi,zcvfi,ra,rg,rd,rcpd)
      print*,'apres iniphysiq'

! 2 PARAMETRES QUI DEVRAIENT ETRE LUS DANS run.def MAIS NE LE SONT PAS ICI:
      co2_ppm= 330.0
      solaire=1370.0
      radpas      = nint(rday/timestep/float(nbapp_rad))

! Ecriture du startphy avant le premier appel a la physique.
! On le met juste avant pour avoir acces a tous les champs
! NB: les clesphy0 seront remplies dans phyredem d'apres les flags lus dans gcm.def

      if (ok_writedem) then

!--------------------------------------------------------------------------
! pbl_surface_init (called here) and pbl_surface_final (called by phyredem)
! need : qsol fder snow qsurf evap rugos agesno ftsoil
!--------------------------------------------------------------------------

        type_ocean = "force"
        run_off_lic_0(1) = restart_runoff 
        call fonte_neige_init(run_off_lic_0)

        fder=0.
        snsrf(1,:)=0.        ! couverture de neige des sous surface
        qsurfsrf(1,:)=qsurf ! humidite de l'air des sous surface
        evap=0.
        frugs(1,:)=rugos     ! couverture de neige des sous surface
        agesno  = xagesno
        tsoil(:,:,:)=tsurf

        call pbl_surface_init(qsol, fder, snsrf, qsurfsrf,
     .       evap, frugs, agesno, tsoil)

        !Ocean force

        print*,'avant phyredem'
        pctsrf(1,:)=0.
        if (nat_surf.eq.0.) then
        pctsrf(1,is_oce)=1.
        pctsrf(1,is_ter)=0.
	else
        pctsrf(1,is_oce)=0.
        pctsrf(1,is_ter)=1.
	end if

	print*,'nat_surf,pctsrf(1,is_oce),pctsrf(1,is_ter)',nat_surf
     $        ,pctsrf(1,is_oce),pctsrf(1,is_ter)

        zmasq=pctsrf(1,is_ter)+pctsrf(1,is_lic)
        zpic = zpicinp
        ftsol=tsurf
        falb1 = albedo                            
        falb2 = albedo                            
        rugoro=rugos 
        t_ancien(1,:)=temp(:)
        q_ancien(1,:)=q(:,1)
        pbl_tke=1.e-8

        rain_fall=0.
        snow_fall=0.
        solsw=0.
        sollw=0.
        radsol=0.
        rnebcon=0.
        ratqs=0.
        clwcon=0.
        zmea=0.
        zstd=0.
        zsig=0.
        zgam=0.
        zval=0. 
        zthe=0. 
        ema_work1=0.
        ema_work2=0.
 
!------------------------------------------------------------------------
! Make file containing restart for the physics (startphy.nc)
!
! NB: List of the variables to be written by phyredem (via put_field):
! rlon,rlat,zmasq,pctsrf(:,is_ter),pctsrf(:,is_lic),pctsrf(:,is_oce)
! pctsrf(:,is_sic),ftsol(:,nsrf),tsoil(:,isoil,nsrf),qsurf(:,nsrf)
! qsol,falb1(:,nsrf),falb2(:,nsrf),evap(:,nsrf),snow(:,nsrf)
! radsol,solsw,sollw,fder,rain_fall,snow_fall,frugs(:,nsrf)
! agesno(:,nsrf),zmea,zstd,zsig,zgam,zthe,zpic,zval,rugoro
! t_ancien,q_ancien,frugs(:,is_oce),clwcon(:,1),rnebcon(:,1),ratqs(:,1)
! run_off_lic_0,pbl_tke(:,1:klev,nsrf),zmax0,f0,ema_work1,ema_work2
! wake_deltat,wake_deltaq,wake_s,wake_cstar,wake_fip
!------------------------------------------------------------------------

        call phyredem ("startphy.nc")

        ok_writedem=.false.
        print*,'apres phyredem'

      endif ! ok_writedem
      
!------------------------------------------------------------------------
! Make file containing boundary conditions (limit.nc)
!
! NB: List of the variables to be written in limit.nc 
!     (by writelim.F, subroutine of 1DUTILS.h):
!        phy_nat,phy_alb,phy_sst,phy_bil,phy_rug,phy_ice,
!        phy_fter,phy_foce,phy_flic,phy_fsic)
!------------------------------------------------------------------------

      do i=1,yd
        phy_nat(i)  = nat_surf
        phy_alb(i)  = albedo
        phy_sst(i)  = tsurf ! read_tsurf1d will be used instead
        phy_rug(i)  = rugos
        phy_fter(i) = pctsrf(1,is_ter)
        phy_foce(i) = pctsrf(1,is_oce)
        phy_fsic(i) = pctsrf(1,is_sic)
        phy_flic(i) = pctsrf(1,is_lic)
      enddo

      call writelim (phy_nat,phy_alb,phy_sst,phy_bil,phy_rug,
     .               phy_ice,phy_fter,phy_foce,phy_flic,phy_fsic)

      call phys_state_var_end

!=====================================================================
! START OF THE TEMPORAL LOOP :
!=====================================================================
           
      do while(it.le.nint(fnday*day_step))

        print*,'XXXXXXXXXXXXXXXXXXX ITAP,day,time=',
     .                                it,day,time,nint(fnday*day_step)
        print*,'PAS DE TEMPS ',timestep

!---------------------------------------------------------------------
! Interpolation TOGA forcing in time and onto model levels
!---------------------------------------------------------------------

      if (forcing_GCSSold) then

       call get_uvd(it,timestep,fich_gcssold_ctl,fich_gcssold_dat,
     :               ht_gcssold,hq_gcssold,hw_gcssold,
     :               hu_gcssold,hv_gcssold,
     :               hthturb_gcssold,hqturb_gcssold,Ts_gcssold,
     :               imp_fcg_gcssold,ts_fcg_gcssold,
     :               Tp_fcg_gcssold,Turb_fcg_gcssold)
       print *,' get_uvd -> hqturb_gcssold ',it,hqturb_gcssold
! large-scale forcing :
!!!      tsurf = ts_gcssold
      do l = 1, llm
!       u(l) = hu_gcssold(l) !  on prescrit le vent
!       v(l) = hv_gcssold(l)    !  on prescrit le vent
!       omega(l) = hw_gcssold(l)
!       rho(l)  = play(l)/(rd*temp(l)*(1.+(rv/rd-1.)*q(l,1)))
!       omega2(l)=-rho(l)*omega(l)
       omega(l) = hw_gcssold(l)
       omega2(l)= omega(l)/rg*airefi ! flxmass_w calcule comme ds physiq

       alpha = rd*temp(l)*(1.+(rv/rd-1.)*q(l,1))/play(l)
       d_th_adv(l) = ht_gcssold(l)
       d_q_adv(l,1) = hq_gcssold(l)
       dt_cooling(l) = 0.0
      enddo

      endif ! forcing_GCSSold

      if (forcing_toga) then

        print*,
     : '#### ITAP,day,day1,(day-day1)*86400,(day-day1)*86400/dt_toga=',
     :    day,day1,(day-day1)*86400.,(day-day1)*86400/dt_toga

! time interpolation:
        CALL interp_toga_time(day,day1,annee_ref
     i             ,year_ini_toga,day_ini_toga,nt_toga,dt_toga,nlev_toga
     i             ,ts_toga,plev_toga,t_toga,q_toga,u_toga,v_toga,w_toga
     i             ,ht_toga,vt_toga,hq_toga,vq_toga
     o             ,ts_prof,plev_prof,t_prof,q_prof,u_prof,v_prof,w_prof
     o             ,ht_prof,vt_prof,hq_prof,vq_prof)

        if (type_ts_forcing.eq.1) ts_cur = ts_prof ! SST used in read_tsurf1d

! vertical interpolation:
      CALL interp_toga_vertical(play,nlev_toga,plev_prof
     :         ,t_prof,q_prof,u_prof,v_prof,w_prof
     :         ,ht_prof,vt_prof,hq_prof,vq_prof
     :         ,t_mod,q_mod,u_mod,v_mod,w_mod
     :         ,ht_mod,vt_mod,hq_mod,vq_mod,mxcalc)

! large-scale forcing :
      tsurf = ts_prof
      do l = 1, llm
       u(l) = u_mod(l) ! sb: on prescrit le vent
       v(l) = v_mod(l) ! sb: on prescrit le vent
!       omega(l) = w_prof(l)
!       rho(l)  = play(l)/(rd*temp(l)*(1.+(rv/rd-1.)*q(l,1)))
!       omega2(l)=-rho(l)*omega(l)
       omega(l) = w_mod(l)
       omega2(l)= omega(l)/rg*airefi ! flxmass_w calcule comme ds physiq

       alpha = rd*temp(l)*(1.+(rv/rd-1.)*q(l,1))/play(l)
       d_th_adv(l) = alpha*omega(l)/rcpd-(ht_mod(l)+vt_mod(l))
       d_q_adv(l,1) = -(hq_mod(l)+vq_mod(l))
       dt_cooling(l) = 0.0
      enddo

      endif ! forcing_toga

      if (forcing_GCM2SCM) then
        write (*,*) 'forcing_GCM2SCM not yet implemented'
        stop 'in time loop'
      endif ! forcing_GCM2SCM

!---------------------------------------------------------------------
!  Geopotential :
!---------------------------------------------------------------------

        phi(1)=RD*temp(1)*(plev(1)-play(1))/(.5*(plev(1)+play(1)))
        do l = 1, llm-1
          phi(l+1)=phi(l)+RD*(temp(l)+temp(l+1))*
     .    (play(l)-play(l+1))/(play(l)+play(l+1))
        enddo

!---------------------------------------------------------------------
!   Call physiq :
!---------------------------------------------------------------------

      print *,' avant physiq : -------------------------------'
      print *,' temp ',temp
      print *,' q ',q
      print *,' omega2 ',omega2

        call physiq(ngrid,llm,
     .              firstcall,lastcall,
     .              day,time,timestep,
     .              plev,play,phi,phis,presnivs,clesphy0,
     .              u,v,temp,q,omega2,
     .              du_phys,dv_phys,dt_phys,dq,dpsrf,
     +              dudyn,PVteta)

        firstcall=.false.
        print*,'APRES PHYS'

        print *,' temp ',temp
        print *,' q ',q
        print *,' dt_phys ',dt_phys

!---------------------------------------------------------------------
!   Add physical tendencies :
!---------------------------------------------------------------------

      fcoriolis=2.*sin(rpi*rlat(1)/180.)*romega

       if (forcing_radconv) then
         fcoriolis=0.0
         dt_cooling=0.0
         d_th_adv=0.0
         d_q_adv=0.0
       endif

       if (forcing_toga .or. forcing_GCSSold) then
         fcoriolis=0.0
       endif

        u(1:mxcalc)=u(1:mxcalc) + timestep*(
     .              du_phys(1:mxcalc)
     .             +fcoriolis*(v(1:mxcalc)-vg(1:mxcalc)))
        v(1:mxcalc)=v(1:mxcalc) + timestep*(
     .	            dv_phys(1:mxcalc)
     .             -fcoriolis*(u(1:mxcalc)-ug(1:mxcalc)))
        q(1:mxcalc,:)=q(1:mxcalc,:)+timestep*(
     .              dq(1:mxcalc,:)
     .             +d_q_adv(1:mxcalc,:))

        print *,
     .    'physiq-> temp(1),dt_phys(1),d_th_adv(1),dt_cooling(1) ',
     .              temp(1),dt_phys(1),d_th_adv(1),dt_cooling(1)

        temp(1:mxcalc)=temp(1:mxcalc)+timestep*(
     .              dt_phys(1:mxcalc)
     .             +d_th_adv(1:mxcalc)
     .             +dt_cooling(1:mxcalc))  ! Taux de chauffage ou refroid.

        teta=temp*(pzero/play)**rkappa

!---------------------------------------------------------------------
!   Add large-scale tendencies (advection, etc) :
!---------------------------------------------------------------------

ccc nrlmd
ccc        tmpvar=teta
ccc        call advect_vert(llm,omega,timestep,tmpvar,plev)
ccc
ccc        teta(1:mxcalc)=tmpvar(1:mxcalc)
ccc        tmpvar(:)=q(:,1)
ccc        call advect_vert(llm,omega,timestep,tmpvar,plev)
ccc        q(1:mxcalc,1)=tmpvar(1:mxcalc)
ccc        tmpvar(:)=q(:,2)
ccc        call advect_vert(llm,omega,timestep,tmpvar,plev)
ccc        q(1:mxcalc,2)=tmpvar(1:mxcalc)

!---------------------------------------------------------------------
!   Air temperature :
!---------------------------------------------------------------------        

        temp=teta*(play/pzero)**rkappa ! samba?

        day =day+1./day_step
        time=day-int(day)
        it=it+1

      enddo

      print*,'Pas de temps final ',it-1
      stop 'The End!'

      end

#include "1DUTILS.h"
#include "1Dconv.h"


